@@ -19,7 +19,8 @@ from .utils import (
     owner_to_address,
     create_tag,
     encode_tag,
-    decode_tag
+    decode_tag,
+    base64url_decode
 )
 from .deep_hash import deep_hash
 from .merkle import compute_root_hash, generate_transaction_chunks
@@ -37,18 +38,28 @@ class ArweaveTransactionException(Exception):
 class Wallet(object):
     HASH = 'sha256'
 
+    def _set_jwk_params(self):
+        self.jwk_data['p2s'] = ''
+        self.jwk = jwk.construct(self.jwk_data, algorithm=jwk.ALGORITHMS.RS256)
+        self.rsa = RSA.importKey(self.jwk.to_pem())
+
+        self.owner = self.jwk_data.get('n')
+        self.address = owner_to_address(self.owner)
+
+        self.api_url = API_URL
+
     def __init__(self, jwk_file='jwk_file.json'):
         with open(jwk_file, 'r') as j_file:
             self.jwk_data = json.loads(j_file.read())
-            self.jwk_data['p2s'] = ''
-            self.jwk = jwk.construct(self.jwk_data, algorithm=jwk.ALGORITHMS.RS256)
-            self.rsa = RSA.importKey(self.jwk.to_pem())
+        self._set_jwk_params()
 
-            self.owner = self.jwk_data.get('n')
-            self.address = owner_to_address(self.owner)
+    @classmethod
+    def from_data(cls, jwk_data):
+        wallet = cls.__new__(cls)
+        wallet.jwk_data = jwk_data
+        wallet._set_jwk_params()
+        return wallet
 
-        self.api_url = API_URL
-       
     @property
     def balance(self):
         url = "{}/wallet/{}/balance".format(self.api_url, self.address)
@@ -57,9 +68,11 @@ class Wallet(object):
 
         if response.status_code == 200:
             balance = winston_to_ar(response.text)
+        else:
+            raise ArweaveTransactionException(response.text)
+
+        return balance
 
-        return balance  
-    
     def sign(self, message):
         h = SHA256.new(message)
         signed_data = PKCS1_PSS.new(self.rsa).sign(h)
@@ -102,7 +115,7 @@ class Transaction(object):
         if type(data) is bytes:
             self.data = base64url_encode(data)
         else:
-            self.data = base64url_encode(data.encode('ascii'))
+            self.data = base64url_encode(data.encode('utf-8'))
 
         if self.data is None:
             self.data = ''
@@ -192,10 +205,10 @@ class Transaction(object):
 
         if int(self.data_size) > 0 and self.data_root == "" and not self.uses_uploader:
             if type(self.data) == str:
-                root_hash = compute_root_hash(io.StringIO(self.data))
+                root_hash = compute_root_hash(io.BytesIO(base64url_decode(self.data.encode('utf-8'))))
 
             if type(self.data) == bytes:
-                root_hash = compute_root_hash(io.BytesIO(self.data))
+                root_hash = compute_root_hash(io.BytesIO(base64url_decode(self.data)))
 
             self.data_root = base64url_encode(root_hash)
 
@@ -244,7 +257,7 @@ class Transaction(object):
         json_data = self.json_data
         response = requests.post(url, data=json_data, headers=headers)
 
-        logger.error("{}\n\n{}".format(response.text, self.json_data))
+        logger.debug("{}\n\n{}".format(response.text, self.json_data))
 
         if response.status_code == 200:
             logger.debug("RESPONSE 200: {}".format(response.text))
@@ -289,7 +302,7 @@ class Transaction(object):
 
         json_str = json.dumps(data)
 
-        logger.error(json_str)
+        logger.debug(json_str)
 
         return json_str.replace(' ', '')
 
