@@ -10,9 +10,11 @@ from .utils import *
 from .merkle import validate_path, CHUNK_SIZE
 from .arweave_lib import API_URL
 
-from signal import signal, SIGPIPE, SIG_DFL
-
-signal(SIGPIPE, SIG_DFL)
+try:
+    from signal import signal, SIGPIPE, SIG_DFL
+    signal(SIGPIPE, SIG_DFL)
+except ImportError:  # If SIGPIPE is not available (win32),
+    pass
 
 logger = logging.getLogger(__name__)
 
@@ -293,6 +295,7 @@ def from_serialized(self, file_handler, json_str):
     upload = TransactionUploader(
         file_handler=file_handler,
         transaction=Transaction(
+            file_handler=file_handler,
             transaction=serialized.transaction
         )
     )
@@ -327,6 +330,7 @@ def from_transaction_id(file_handler, transaction_str, wallet, api_url=API_URL):
         file_handler=file_handler,
         transaction=Transaction(
             wallet,
+            file_handler=file_handler,
             transaction=response.text
         )
     )
